"use strict";
// Copyright 2021 The Casbin Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CanVisitType = exports.CanDirective = void 0;
const utils_1 = require("@graphql-tools/utils");
const graphql_1 = require("graphql");
const utils_2 = require("./utils");
const error_1 = require("./error");
/**
 * Apply directive to paticular field.
 */
class CanDirective extends utils_1.SchemaDirectiveVisitor {
    /**
     * internal function, be called to get directive info.
     * You should also implicit define directive in the GraphQL SDL after graphql-js 14.0.0
     * @param directiveName
     * @param schema
     */
    static getDirectiveDeclaration(directiveName, schema) {
        return new graphql_1.GraphQLDirective({
            name: 'can',
            locations: [graphql_1.DirectiveLocation.FIELD_DEFINITION],
            args: {
                who: {
                    type: graphql_1.GraphQLString,
                    defaultValue: '*',
                },
            },
        });
    }
    /**
     * internal function, be called to visit fields.
     * @param field
     */
    visitFieldDefinition(field) {
        const { resolve = graphql_1.defaultFieldResolver } = field;
        const { who } = this.args;
        if (field.type instanceof graphql_1.GraphQLNonNull && field.type.ofType instanceof graphql_1.GraphQLScalarType) {
            field.type = new graphql_1.GraphQLNonNull(new CanVisitType(field.type.ofType));
        }
        else if (field.type instanceof graphql_1.GraphQLScalarType) {
            field.type = new CanVisitType(field.type);
        }
        else {
            throw new Error(`Not a scalar type: ${field.type}`);
        }
        field.resolve = function (source, args, context, info) {
            return __awaiter(this, void 0, void 0, function* () {
                const action = info.operation.operation;
                const path = utils_2.Info2Path(info);
                let role = context[utils_2.CasbinContextRoleKey];
                if (!(utils_2.CasbinContextEnforcerKey in context)) {
                    throw new error_1.EnforcerNotFoundError(`You must provide Enforcer in the context with key ${utils_2.CasbinContextEnforcerKey}`);
                }
                const CasbinInstance = context[utils_2.CasbinContextEnforcerKey];
                if (!(utils_2.CasbinContextPathCacheKey in context)) {
                    context[utils_2.CasbinContextPathCacheKey] = yield utils_2.buildPathCache(CasbinInstance);
                }
                const pathCache = context[utils_2.CasbinContextPathCacheKey];
                if (!pathCache.includes(path)) {
                    yield CasbinInstance.addPolicy(who, path, action);
                    pathCache.push(utils_2.PathAndAction2Key(path, action));
                }
                const passed = yield CasbinInstance.enforce(role, path, action);
                if (passed) {
                    return yield resolve.apply(this, [source, args, context, info]);
                }
                else {
                    if (role === '*') {
                        role = 'anonymous';
                    }
                    throw new error_1.PermissionInsufficientError(`${role} can not ${action} ${path}`);
                }
            });
        };
    }
}
exports.CanDirective = CanDirective;
/**
 * Override GraphQLScalarType, used to judge if current node have directive.
 */
class CanVisitType extends graphql_1.GraphQLScalarType {
    constructor(type) {
        super({
            name: utils_2.CasbinAuthTypeName,
            serialize(value) {
                return type.serialize(value);
            },
            parseLiteral(value, maybe) {
                return type.parseLiteral(value, maybe);
            },
            parseValue(value) {
                return type.parseValue(value);
            },
        });
    }
}
exports.CanVisitType = CanVisitType;
