"use strict";
// Copyright 2021 The Casbin Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.newMiddleware = exports.MiddlewareOptions = void 0;
const utils_1 = require("./utils");
const error_1 = require("./error");
const graphql_1 = require("graphql");
/**
 * node-casbin instance and the key of graphql context to get `role`.
 */
class MiddlewareOptions {
    constructor() {
        this.ctxMember = 'role';
    }
}
exports.MiddlewareOptions = MiddlewareOptions;
/**
 * newMiddleware return a middleware for graphql-middleware
 * @param options node-casbin instance and the key of graphql context to get `role`.
 */
const newMiddleware = (options) => __awaiter(void 0, void 0, void 0, function* () {
    const CasbinInstance = options.enforcer;
    return (resolve, root, args, context, info) => __awaiter(void 0, void 0, void 0, function* () {
        let role = context[options.ctxMember] ? context[options.ctxMember] : '*';
        const path = utils_1.Info2Path(info);
        const action = info.operation.operation;
        if (!(utils_1.CasbinContextRoleKey in context)) {
            context[utils_1.CasbinContextRoleKey] = role;
        }
        if (!(utils_1.CasbinContextPathCacheKey in context)) {
            context[utils_1.CasbinContextPathCacheKey] = yield utils_1.buildPathCache(CasbinInstance);
        }
        const pathCache = context[utils_1.CasbinContextPathCacheKey];
        const key = utils_1.PathAndAction2Key(path, action);
        // Try to find if exist directive.
        // Will skip path check to execute directive query.
        const returnType = info.returnType;
        if (!pathCache.includes(key)) {
            if ((returnType instanceof graphql_1.GraphQLScalarType && returnType.name === utils_1.CasbinAuthTypeName) ||
                (returnType instanceof graphql_1.GraphQLNonNull &&
                    returnType.ofType instanceof graphql_1.GraphQLScalarType &&
                    returnType.ofType.name === utils_1.CasbinAuthTypeName)) {
                // wait for directive
                return yield resolve(root, args, context, info);
            }
            else {
                throw new error_1.RuleNotFoundError(`Can not find rule for ${action} ${path}`);
            }
        }
        const passed = yield CasbinInstance.enforce(role, path, action);
        if (passed) {
            return yield resolve(root, args, context, info);
        }
        else {
            if (role === '*') {
                role = 'anonymous';
            }
            throw new error_1.PermissionInsufficientError(`${role} can not ${action} ${path}`);
        }
    });
});
exports.newMiddleware = newMiddleware;
