"use strict";
// Copyright 2021 The Casbin Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildPathCache = exports.PathAndAction2Key = exports.Info2Path = exports.CasbinAuthTypeName = exports.CasbinContextRoleKey = exports.CasbinContextEnforcerKey = exports.CasbinContextPathCacheKey = void 0;
exports.CasbinContextPathCacheKey = '_casbinPathCache';
exports.CasbinContextEnforcerKey = '_casbinEnforcer';
exports.CasbinContextRoleKey = '_casbinRole';
exports.CasbinAuthTypeName = 'CasbinCanVisit';
/**
 * Get the full path of current endpoint by GraphQLResolveInfo.
 * @param info GraphQLResolveInfo
 */
const Info2Path = (info) => {
    let node = info.path;
    let fullPath = node.key.toString();
    while (node.prev) {
        node = node.prev;
        if (typeof node.key !== 'number') {
            fullPath = node.key.toString() + '.' + fullPath;
        }
    }
    return fullPath;
};
exports.Info2Path = Info2Path;
/**
 * Create key with endpoint path and action.
 * @param path
 * @param action
 */
const PathAndAction2Key = (path, action) => {
    return `${path}_${action}`;
};
exports.PathAndAction2Key = PathAndAction2Key;
/**
 * use policies to build path cache. This can get additional performance optimize.
 * @param CasbinInstance
 */
const buildPathCache = (CasbinInstance) => __awaiter(void 0, void 0, void 0, function* () {
    const CasbinGraphQLCache = [];
    const policies = yield CasbinInstance.getPolicy();
    for (const policy of policies) {
        const key = exports.PathAndAction2Key(policy[1], policy[2]);
        if (!CasbinGraphQLCache.includes(key)) {
            CasbinGraphQLCache.push(key);
        }
    }
    return CasbinGraphQLCache;
});
exports.buildPathCache = buildPathCache;
